import java.awt.*;
import java.awt.geom.*;

class GInverter extends Inverter implements Drawable, GOutSignal, GInSignal {
   private static final Path2D.Double polyline = 
                      new Path2D.Double(Path2D.WIND_EVEN_ODD,11);
   private static final Rectangle2D.Double input= new Rectangle2D.Double(-5,11,10,10);
   private static final Rectangle2D.Double output= new Rectangle2D.Double(75,11,10,10);
   static {  // static initialization block. 
      polyline.moveTo (16, 0);
      polyline.lineTo(16,16);
      polyline.lineTo(0,16);
      polyline.lineTo(16,16);
      polyline.lineTo(16,32);
      polyline.lineTo(56,16);
      polyline.curveTo(56,23, 66,23, 66,16);
      polyline.lineTo(80,16);
      polyline.lineTo(66,16);
      polyline.curveTo(66,9, 56,9, 56,16);
      polyline.lineTo(16,0);
   }
   private Color color = Color.BLUE;
   private Path2D.Double shape;
   private Path2D.Double in;
   private Path2D.Double out;
   private final BasicStroke stroke;
   
   public GInverter(Point2D.Double pos, Simulator sim){
      super(sim);
      AffineTransform at = AffineTransform.getTranslateInstance(pos.x, pos.y);
      shape = (Path2D.Double) at.createTransformedShape(polyline);
      in = (Path2D.Double) at.createTransformedShape(input);
      out = (Path2D.Double) at.createTransformedShape(output);
      stroke = new BasicStroke(2);
   }
   public void draw (Graphics2D g){
      g.setColor(color);
      g.setStroke(stroke);
      g.draw(shape);
      if (hasConnectedSource())
        g.fill(in);
      else
        g.draw(in);
      g.draw(out);
   }
   public boolean contains (Point p){
     return shape.contains(p);
   }
   
   public void dragIn(int dx, int dy) {
      AffineTransform at = AffineTransform.getTranslateInstance(dx, dy);
      shape.transform(at);
      in.transform(at);
      out.transform(at);
      disconnectInput();
      disconnectAllOutput();
   }
   public void flip() {
     AffineTransform af = new AffineTransform();   
     Rectangle2D box = shape.getBounds2D(); 
     double x = box.getX()+box.getWidth()/2;

	  // Note: the transformations get applied IN REVERSE ORDER!
	  af.translate(x,0);
	  af.scale(-1, 1);
	  af.translate(-x, 0);          
	  shape.transform(af);
     in.transform(af);
     out.transform(af);
   }
   public void setSelected() {
     color = Color.RED;
   }
   public void setReleased(){
     color = Color.BLUE;
   }
   public Shape getGOutput() {
     return out;
   }
   public Shape getGInput() {
     return in;
   }
   public void delete(){
		disconnectInput();
		disconnectAllOutput();
		AffineTransform del = new AffineTransform();   
		del.translate(-1000,-1000);
		shape.transform(del);
		in.transform(del);
		out.transform(del);
   }
}