import javax.swing.JPanel;
import javax.swing.JOptionPane;
import java.awt.event.*;
import java.awt.event.MouseAdapter;
import java.awt.*;

public class MouseListener extends MouseAdapter implements KeyListener{
   private DigitalCircuit circuit;
   private Drawable currentComponent;
   private JPanel panel;
   private Point last, current;
   public MouseListener (DigitalCircuit c, JPanel p){
      circuit = c;
      panel = p;
      last = current = null;
   } 
   public void mouseMoved(MouseEvent e) {
      last = e.getPoint();
      Drawable newComponent = circuit.find(last);
      if (newComponent == currentComponent) return;
      if (currentComponent != null) {
         currentComponent.setReleased();
         currentComponent = null;
      }
      if (newComponent != null) { 
         currentComponent = newComponent;
         currentComponent.setSelected();
      }
      panel.repaint();
   }
   public void mouseDragged(MouseEvent e) {
      if (currentComponent == null) return;
      current = e.getPoint();
      currentComponent.dragIn(current.x-last.x, current.y-last.y);
      last=current;
      panel.repaint();
   }
   public void mouseWheelMoved(MouseWheelEvent e) {
      if (currentComponent == null) return;
      currentComponent.flip();
      panel.repaint();
   }
   public void mouseReleased(MouseEvent e) {
      if (currentComponent instanceof GWire) {
         GWire w = (GWire)currentComponent;
         w.setReleased();
         w.tryToConnect();         
         panel.repaint();
      }
   }
	public void keyPressed(KeyEvent e){
		int key = e.getKeyCode();
		if(currentComponent != null){
			if(key == KeyEvent.VK_D){
					currentComponent.delete();
					System.out.println("DELETE");
					panel.repaint();

			}
			else if(key == KeyEvent.VK_R){
				if (currentComponent instanceof Gate){
					try {
						String s = JOptionPane.showInputDialog("Ingrese nuevo retardo");
						float d = Float.parseFloat(s);
						((Gate)currentComponent).setDelay(d);
					}
					catch ( Exception err){
						JOptionPane.showMessageDialog(null, "Error al introducir el retardo");
						return;
					}
					
				}
			}
		}
	}
	public void keyReleased(KeyEvent e){
		return;
	}
	public void keyTyped(KeyEvent e){
		return;
	}
}